import {
  DEFAULT_FILES,
  DEFAULT_EDITOR_THEME,
  DEFAULT_EDITOR_LANGUAGE,
} from "@/config";
import {
  Select,
  SelectContent,
  SelectItem,
  SelectTrigger,
  SelectValue,
} from "@/components/ui/select";
import {
  Copy,
  FileDown,
  Paintbrush,
  Palette,
  Redo2,
  Undo2,
} from "lucide-react";
import * as monaco from "monaco-editor";
import { highlighter } from "@/lib/shiki";
import { Bot, CodeXml } from "lucide-react";
import { shikiToMonaco } from "@shikijs/monaco";
import { useEffect, useRef, useState } from "react";
import { SUPPORTED_EDITOR_THEMES } from "@/constants/themes";
import { ScrollArea, ScrollBar } from "@/components/ui/scroll-area";
import { DiffEditor, Editor, Monaco, loader } from "@monaco-editor/react";
import { SUPPORTED_EDITOR_LANGUAGES_CONFIG } from "@/constants/languages";
import editorWorker from "monaco-editor/esm/vs/editor/editor.worker?worker";
import { connectToLanguageServer, SUPPORTED_LSP_LANGUAGES } from "@/lib/lsp";
import { Tabs, TabsContent, TabsList, TabsTrigger } from "@/components/ui/tabs";
import { Button } from "@/components/ui/button";
import {
  Tooltip,
  TooltipContent,
  TooltipProvider,
  TooltipTrigger,
} from "@/components/ui/tooltip";
import {
  redoChange,
  undoChange,
  formatCode,
  downloadCode,
  copyCode,
  moveCursorToEnd,
} from "@/lib/editor";

self.MonacoEnvironment = {
  getWorker(_, _label) {
    return new editorWorker();
  },
};

loader.config({ monaco });

function App() {
  const [language, setLanguage] = useState(DEFAULT_EDITOR_LANGUAGE);
  const [theme, setTheme] = useState(DEFAULT_EDITOR_THEME);
  const file = DEFAULT_FILES[language];
  const webSocketRef = useRef<WebSocket | null>(null);
  const editorRef = useRef<monaco.editor.IStandaloneCodeEditor | null>(null);

  useEffect(() => {
    const closeWebSocket = () => {
      if (webSocketRef.current) {
        if (
          webSocketRef.current.readyState === WebSocket.OPEN ||
          webSocketRef.current.readyState === WebSocket.CONNECTING
        ) {
          webSocketRef.current.onclose = () => {
            webSocketRef.current = null;
          };
          webSocketRef.current.close();
        } else {
          webSocketRef.current = null;
        }
      }
    };

    const connectWebSocket = async () => {
      closeWebSocket();

      if (editorRef.current && language in SUPPORTED_LSP_LANGUAGES) {
        try {
          const webSocket = await connectToLanguageServer(language);
          webSocket.onopen = () => {
            webSocketRef.current = webSocket;
          };
        } catch (error) {
          console.error("Failed to connect to language server", error);
        }
      }
    };

    connectWebSocket();
    return closeWebSocket;
  }, [language]);

  return (
    <div className="h-screen bg-[#282c34] dark overflow-hidden">
      <Tabs defaultValue="code-editor" className="h-full mt-2">
        <ScrollArea className="border-b border-[#3e4452]">
          <div className="flex items-center justify-between">
            <TabsList className="gap-1 bg-transparent px-4 mb-2">
              <TabsTrigger
                value="code-editor"
                className="rounded-full data-[state=active]:bg-primary data-[state=active]:text-primary-foreground data-[state=active]:shadow-none"
              >
                <CodeXml
                  className="-ms-0.5 me-1.5 opacity-60"
                  size={16}
                  strokeWidth={2}
                  aria-hidden="true"
                />
                代码
              </TabsTrigger>
            </TabsList>
            <div className="flex items-center gap-x-2 pr-4 mb-2">
              <Select value={theme} onValueChange={setTheme}>
                <SelectTrigger className="relative ps-9 w-40 bg-foreground">
                  <div className="pointer-events-none absolute inset-y-0 start-0 flex items-center justify-center ps-3 text-muted-foreground/80 group-has-[[disabled]]:opacity-50">
                    <Palette size={16} strokeWidth={2} aria-hidden="true" />
                  </div>
                  <SelectValue placeholder="Select Theme" />
                </SelectTrigger>
                <SelectContent>
                  {SUPPORTED_EDITOR_THEMES.map((theme) => (
                    <SelectItem key={theme.id} value={theme.id}>
                      <span className="truncate">{theme.label}</span>
                    </SelectItem>
                  ))}
                </SelectContent>
              </Select>
              <Select value={language} onValueChange={setLanguage}>
                <SelectTrigger className="[&>span]:flex [&>span]:items-center [&>span]:gap-2 [&>span_svg]:shrink-0 [&>span_svg]:text-muted-foreground/80 w-36 bg-foreground">
                  <SelectValue placeholder="Select Language" />
                </SelectTrigger>
                <SelectContent className="[&_*[role=option]>span>svg]:shrink-0 [&_*[role=option]>span>svg]:text-muted-foreground/80 [&_*[role=option]>span]:end-2 [&_*[role=option]>span]:start-auto [&_*[role=option]>span]:flex [&_*[role=option]>span]:items-center [&_*[role=option]>span]:gap-2 [&_*[role=option]]:pe-8 [&_*[role=option]]:ps-2">
                  {Object.values(SUPPORTED_EDITOR_LANGUAGES_CONFIG).map(
                    (language) => (
                      <SelectItem key={language.id} value={language.id}>
                        <language.icon size={16} aria-hidden="true" />
                        <span className="truncate">{language.label}</span>
                      </SelectItem>
                    )
                  )}
                </SelectContent>
              </Select>
            </div>
          </div>
          <ScrollBar orientation="horizontal" />
        </ScrollArea>
        <TabsContent value="code-editor" className="h-full mt-0">
          <div className="flex items-center justify-end gap-x-2 pr-4">
            <TooltipProvider delayDuration={0}>
              <Tooltip>
                <TooltipTrigger asChild>
                  <Button
                    variant="outline"
                    size="icon"
                    aria-label="Redo"
                    className="bg-white"
                    onClick={() => redoChange(editorRef.current!)}
                  >
                    <Redo2 size={16} strokeWidth={2} aria-hidden="true" />
                  </Button>
                </TooltipTrigger>
                <TooltipContent className="px-2 py-1 text-xs">
                  重做
                </TooltipContent>
              </Tooltip>
            </TooltipProvider>
            <TooltipProvider delayDuration={0}>
              <Tooltip>
                <TooltipTrigger asChild>
                  <Button
                    variant="outline"
                    size="icon"
                    aria-label="Undo"
                    className="bg-white"
                    onClick={() => undoChange(editorRef.current!)}
                  >
                    <Undo2 size={16} strokeWidth={2} aria-hidden="true" />
                  </Button>
                </TooltipTrigger>
                <TooltipContent className="px-2 py-1 text-xs">
                  撤销
                </TooltipContent>
              </Tooltip>
            </TooltipProvider>
            <TooltipProvider delayDuration={0}>
              <Tooltip>
                <TooltipTrigger asChild>
                  <Button
                    variant="outline"
                    size="icon"
                    aria-label="Format"
                    className="bg-white"
                    onClick={() => formatCode(editorRef.current!)}
                  >
                    <Paintbrush size={16} strokeWidth={2} aria-hidden="true" />
                  </Button>
                </TooltipTrigger>
                <TooltipContent className="px-2 py-1 text-xs">
                  格式化
                </TooltipContent>
              </Tooltip>
            </TooltipProvider>
            <TooltipProvider delayDuration={0}>
              <Tooltip>
                <TooltipTrigger asChild>
                  <Button
                    variant="outline"
                    size="icon"
                    aria-label="Copy"
                    className="bg-white"
                    onClick={() => copyCode(editorRef.current!)}
                  >
                    <Copy size={16} strokeWidth={2} aria-hidden="true" />
                  </Button>
                </TooltipTrigger>
                <TooltipContent className="px-2 py-1 text-xs">
                  复制
                </TooltipContent>
              </Tooltip>
            </TooltipProvider>
            <TooltipProvider delayDuration={0}>
              <Tooltip>
                <TooltipTrigger asChild>
                  <Button
                    variant="outline"
                    size="icon"
                    aria-label="Download"
                    className="bg-white"
                    onClick={() => downloadCode(editorRef.current!)}
                  >
                    <FileDown size={16} strokeWidth={2} aria-hidden="true" />
                  </Button>
                </TooltipTrigger>
                <TooltipContent className="px-2 py-1 text-xs">
                  下载
                </TooltipContent>
              </Tooltip>
            </TooltipProvider>
          </div>
          <Editor
            theme={theme}
            path={file.path}
            defaultLanguage={file.language}
            defaultValue={file.value}
            options={{
              minimap: { enabled: false },
              fontSize: 16,
              tabSize: 2,
              showFoldingControls: "always",
              automaticLayout: true,
              autoIndent: "full",
              guides: {
                bracketPairs: true,
                indentation: true,
              },
              padding: { top: 8 },
            }}
            beforeMount={async (monaco: Monaco) => {
              shikiToMonaco(await highlighter, monaco);
            }}
            onMount={(editor: monaco.editor.IStandaloneCodeEditor) => {
              editorRef.current = editor;
              const value = editor.getModel()?.getValue();
              if (value !== undefined) {
                window.parent.postMessage(
                  { type: "editor-info", language, code: value },
                  "*"
                );
              }
              if (language in SUPPORTED_LSP_LANGUAGES) {
                connectToLanguageServer(language).then((webSocket) => {
                  webSocketRef.current = webSocket;
                });
              }
            }}
            onChange={(value) => {
              if (value !== undefined) {
                window.parent.postMessage(
                  { type: "editor-info", language, code: value },
                  "*"
                );
              }
            }}
          />
        </TabsContent>
      </Tabs>
    </div>
  );
}

export default App;
