import {
  toSocket,
  WebSocketMessageReader,
  WebSocketMessageWriter,
} from "vscode-ws-jsonrpc";
import {
  CloseAction,
  ErrorAction,
  MessageTransports,
} from "vscode-languageclient";
import normalizeUrl from "normalize-url";
import { MonacoLanguageClient } from "monaco-languageclient";

export const SUPPORTED_LSP_LANGUAGES: {
  [key: string]: { hostname: string; port: number | null; path: string };
} = {
  c: {
    hostname: "c.litchi.icu",
    port: null,
    path: "/clangd",
  },
  cpp: {
    hostname: "cpp.litchi.icu",
    port: null,
    path: "/clangd",
  },
};

function createUrl(
  hostname: string,
  port: number | null,
  path: string
): string {
  const protocol = location.protocol === "https:" ? "wss" : "ws";
  return port !== null
    ? normalizeUrl(`${protocol}://${hostname}:${port}${path}`)
    : normalizeUrl(`${protocol}://${hostname}${path}`);
}

function createLanguageClient(
  transports: MessageTransports
): MonacoLanguageClient {
  return new MonacoLanguageClient({
    name: "Judge4c Language Client",
    clientOptions: {
      documentSelector: ["c", "cpp"],
      errorHandler: {
        error: () => ({ action: ErrorAction.Continue }),
        closed: () => ({ action: CloseAction.DoNotRestart }),
      },
    },
    connectionProvider: {
      get: () => {
        return Promise.resolve(transports);
      },
    },
  });
}

function createWebSocket(url: string) {
  const webSocket = new WebSocket(url);
  webSocket.onopen = () => {
    const socket = toSocket(webSocket);
    const reader = new WebSocketMessageReader(socket);
    const writer = new WebSocketMessageWriter(socket);
    const languageClient = createLanguageClient({
      reader,
      writer,
    });
    languageClient.start();
    reader.onClose(() => languageClient.stop());
  };
  return webSocket;
}

export async function connectToLanguageServer(
  language: string
): Promise<WebSocket> {
  const { hostname, port, path } = SUPPORTED_LSP_LANGUAGES[language];
  const url = createUrl(hostname, port, path);
  return createWebSocket(url);
}
